$DEBUG
C Reads <project_name>.cor correlation file (format as used in 
C PQMethod software) and outputs <project_name>.unr with Centroid
C loading matrix, and listing file <project_name>.lis with iterative 
C sets of communality estimates, the final factors, and the residual
C correlation matrix (see Horst, 1965, p.129ff.)
C Adapted from
C  Horst, Paul (1965): Factor analysis of data matrices. New York:
C  Holt, Rinehart and Winston.
C  CHAPTER 5 - THE CENTROID METHOD
C    5.5 ITERATIVE SOLUTIONS FOR DIAGONALS
C 
C Modifications of original algorithm:
C (1) Introduced upper limit of no of factors (NFAX) extracted. NFAX
C     is read from first header record of correlation file, and must
C     not exceed 8.
C (2) Uncomment loop 12 to replace initial 1.0 diagonals with largest 
C     off-diagonal coefficient. (not much difference, anyway)
C
      DIMENSION R(120,120),W(120),V(120),D(120),U(120),F(120,8),
     x          HOLDR(120,120)
      CHARACTER NAME*8, TITLE*68, BUF*66
C
      WRITE (*,*) ' Enter a One Word Project Name: '
      READ (*,'(A)') NAME

      OPEN (15,FILE=NAME(1:LENGTH(NAME))//'.cor')
      OPEN (16,FILE=NAME(1:LENGTH(NAME))//'.lis')
      OPEN (17,FILE=NAME(1:LENGTH(NAME))//'.unr')
C
      P=.001
      NL=30
      READ (15, 03) NFAX, N, M, TITLE, BUF
 03   FORMAT (3I3,1X,A/A)
      IF (NFAX.LE.0) NFAX=8
      NFAX=MIN(NFAX,8)
      DO 05  I=1,N
 05   READ (15, 06) (HOLDR(I,J), J=1,N)
 06   FORMAT (8(F9.5,1X))
      FN = N
      FM = M
      DO 11 I=1,N
      U(I) = HOLDR(I,I)
      DO 11 J=1,N
      R(I,J)=HOLDR(I,J)
 11   CONTINUE
      L = 0
      K = 0
      KF = 0
C12C  Start with largest off diagonal
C12      DO 12 J=1,N
C12      R(J,J)=0.
C12      DO 12 I=1,N
C12 12   IF(I.NE.J) R(J,J)=MAX(R(J,J),ABS(R(I,J)))
C     TEST SIGNIFICANCE OF RESIDUAL
 15   DO 17  I=1,N
      D(I) = R(I,I)
 17   R(I,I) =0.0
      S = 0.0
      DO 21  I=1,N
      DO 21  J=1,N
 21   S = S + R(I,J)**2
      IF ((2.0*S*FM) - (FN*(FN-1.0)).LE.0) GOTO 24
C      IF ((2.*S/FN/(FN-1)) .LE. 0.01) GOTO 24
      IF (K.GE.NFAX) GOTO 24
      GOTO 25
 24   IF (K-1) 93, 93, 54
C     SIGN VECTOR
 25   DO 29  I=1,N
      W(I) = 0.0
      V(I)=1.0
      DO 29  J=1,N
 29   W(I) = W(I) + R(I,J)
 30   J = 1
      DO 34  I=2,N
      IF (W(J) * V(J) - W(I) * V(I)) 34, 33, 33
 33   J = I
 34   CONTINUE
      IF (W(J) * V(J)) 36, 40, 40
 36   V(J) = -V(J)
      DO 38  I=1,N
 38   W(I) = W(I) + 2.0 * R(I,J) * V(J)
      GO TO 30
C     FACTOR VECTOR
 40   Do 41  I = 1,N
 41   W(I) = W(I) + D(I) * V(I)
      S = 0.0
      DO 44  I =1,N
 44   S = S + ABS(W(I))
      S = 1.0/SQRT(S)
      DO 47  I=1,N
 47   W(I) = W(I) * S
      K = K+1
      DO 48 I=1,N
 48   F(I,K)=W(I)
C     RESIDUAL MATRIX
      DO 52  I=1,N
      R(I,I) = D(I)
      DO 52  J=1,N
 52   R(I,J) = R(I,J) - W(I) * W(J)
      GO TO 15
C     DIAGONALS OF R
 54   S = 0.0
      DO 542 I = 1,N
 542  S = MAX(S,ABS(D(I)))
      DO 56 I =1,N
      D(I) = MIN(1.0,(U(I) - D(I)))
 56   U(I) = D(I)
      WRITE (16,71) (U(I),I=1,N)
 71   FORMAT (15F7.4)
      IF (S-P) 83, 83, 73
 73   L = L+1
      WRITE (16, 732) K,L
 732  FORMAT (2I4)
C      IF (L-NL) 75, 75, 83
      IF (L.GT.NL) THEN
        WRITE (*,'(A,I4,A)') 
     X  ' Communality estimates did NOT converge after',NL,
     X  ' Iterations'
        WRITE (*,*) ' Press <ENTER> to continue'
        READ (*,*)
        GOTO 83
      ENDIF
C     NEW DIAGONALS
 75   K = 0
      DO 77  I=1,N
      DO 77 J=1,N
      R(I,J)=HOLDR(I,J)
 77   CONTINUE
      DO 81  I=1,N
 81   R(I,I) = 0.0
      GO TO 25
 83   WRITE (16, 84)
 84   FORMAT (1H )
      DO 88  J=1,K
      WRITE (16, 71) (F(I,J), I=1,N)
 88   CONTINUE
      WRITE (17, 03) K, N, M, TITLE, BUF
      WRITE (16, 84)
      DO 92  I=1,N
      WRITE (17, '(8(F9.5,1X))') (F(I,J),J=1,K)
 92   WRITE (16, 71) (R(I,J), J=1,N)
 93   CONTINUE
      STOP
      END 
C
      FUNCTION LENGTH (CHR)
C
C    Length of a string without trailing blanks
C
      CHARACTER*(*) CHR
      DO 10 I=LEN(CHR),1,-1
 10   IF(CHR(I:I).NE.' ') GOTO 19
 19   LENGTH=MAX(1,I)
      END
